﻿using System;
using System.Text;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Shapes;

namespace QSFP_DD_LPBK
{
    /// <summary>
    /// Interaction logic for IPConfiguration.xaml
    /// </summary>
    public partial class IPConfiguration : Window
    {
        Wrapper wrapper;
        int instance = 0;
        public bool restart;
        public IPConfiguration(Wrapper wrapper, ref bool Restart)
        {
            try
            {
                InitializeComponent();
                this.wrapper = wrapper;
                this.restart = Restart;

                bt_ReadAll_Click(null, null);
            }

            catch (Exception ex)
            {
                wrapper.exceptionLog(ex);
            }
        }

        private bool CheckIPIntegrity(string Ip)
        {
            try
            {
                string[] IPSelectors = Ip.Split(new char[] { '.' });
                if (IPSelectors.Length != 4)
                {
                    return false;
                }
                int[] subIpValues = new int[4];
                for (int i = 0; i < 4; i++)
                {
                    if (int.TryParse(IPSelectors[i], out subIpValues[i]) == true)
                    {
                        if (subIpValues[i] < 0 || subIpValues[i] > 255)
                        {
                            return false;
                        }
                    }
                    else
                    {
                        return false;
                    }
                }

                return true;
            }

            catch (Exception ex)
            {
                wrapper.exceptionLog(ex);
                return false;
            }
        }

        private void bt_ChangeIP_Click(object sender, RoutedEventArgs e)
        {
            try
            {
                if (tb_IP.Text == string.Empty)
                {
                    MessageBox.Show("Invalid IP", "Error", MessageBoxButton.OK, MessageBoxImage.Error);
                    return;
                }
                string ip = tb_IP.Text;
                if (!CheckIPIntegrity(ip))
                {
                    MessageBox.Show("Invalid IP", "Error", MessageBoxButton.OK, MessageBoxImage.Error);
                    return;
                }
                if (wrapper.ChangeIPConfiguration(instance, ip))
                {
                    restart = true;
                    MessageBox.Show("You need to restart the board for the changes to take effect", "Restart", MessageBoxButton.OK, MessageBoxImage.Information);
                    return;
                }
            }
            catch (Exception ex)
            {
                wrapper.exceptionLog(ex);
            }
        }

        private void bt_ReadIP_Click(object sender, RoutedEventArgs e)
        {
            try
            {
                string ip = "";
                wrapper.readIPConfiguration(instance,ref ip);
                tb_IP.Text = ip;
            }
            catch (Exception ex)
            {
                wrapper.exceptionLog(ex);
            }
        }

        private void bt_ChangeMask_Click(object sender, RoutedEventArgs e)
        {
            try
            {
                if (tb_Mask.Text == string.Empty)
                {
                    MessageBox.Show("Invalid Mask", "Error", MessageBoxButton.OK, MessageBoxImage.Error);
                    return;
                }
                string Mask = tb_Mask.Text;
                if (!CheckIPIntegrity(Mask))
                {
                    MessageBox.Show("Invalid IP", "Error", MessageBoxButton.OK, MessageBoxImage.Error);
                    return;
                }
                if (wrapper.SetEthIPMask(instance,Mask))
                {
                    restart = true;
                    MessageBox.Show("You need to restart the board for the changes to take effect", "Restart", MessageBoxButton.OK, MessageBoxImage.Information);
                    return;
                }
            }
            catch (Exception ex)
            {
                wrapper.exceptionLog(ex);
            }
        }

        private void bt_readMask_Click(object sender, RoutedEventArgs e)
        {
            try
            {
                string Mask = "";
                wrapper.readIPMask(instance,ref Mask);
                tb_Mask.Text = Mask;
            }
            catch (Exception ex)
            {
                wrapper.exceptionLog(ex);
            }
        }

        private void bt_ChangeGatw_Click(object sender, RoutedEventArgs e)
        {
            try
            {
                if (tb_Gatew.Text == string.Empty)
                {
                    restart = true;
                    MessageBox.Show("Invalid Gateway", "Error", MessageBoxButton.OK, MessageBoxImage.Error);
                    return;
                }
                string Gatw = tb_Gatew.Text;
                if (!CheckIPIntegrity(Gatw))
                {
                    MessageBox.Show("Invalid IP", "Error", MessageBoxButton.OK, MessageBoxImage.Error);
                    return;
                }
                if (wrapper.SetEthIPGateway(instance,Gatw))
                {
                    MessageBox.Show("You need to restart the board for the changes to take effect", "Restart", MessageBoxButton.OK, MessageBoxImage.Information);
                    return;
                }
            }
            catch (Exception ex)
            {
                wrapper.exceptionLog(ex);
            }
        }

        private void bt_readGatw_Click(object sender, RoutedEventArgs e)
        {
            try
            {
                string Gatw = "";
                wrapper.readGateway(instance,ref Gatw);
                tb_Gatew.Text = Gatw;
            }
            catch (Exception ex)
            {
                wrapper.exceptionLog(ex);
            }
        }

        private void bt_ChangeAll_Click(object sender, RoutedEventArgs e)
        {
            try
            {
                bt_ChangeIP_Click(sender, e);
                bt_ChangeMask_Click(sender, e);
                bt_ChangeGatw_Click(sender, e);
            }
            catch (Exception ex)
            {
                wrapper.exceptionLog(ex);
            }
        }

        private void bt_ReadAll_Click(object sender, RoutedEventArgs e)
        {
            try
            {
                bt_ReadIP_Click(sender, e);
                bt_readMask_Click(sender, e);
                bt_readGatw_Click(sender, e);
            }
            catch (Exception ex)
            {
                wrapper.exceptionLog(ex);
            }
        }
    }
}
