﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading;

namespace QSFP_DD_LPBK
{
    class MSAManager
    {
        public ConnectionManager cn;
        UInt16 tempPort = 0;
        byte[] tempReadBuff1 = new byte[32];
        byte[] tempReadBuff2 = new byte[32];
        string ReadString = "";
        string asciiResult = "";
        string tempReadString;
        int tempInterval;
        float tempFloat;
        int tempInt;
        public UInt16 MSAInstance = 0;
        public int BoardID=0;

        /// <summary>
        /// Constructor
        /// </summary>
        public MSAManager()
        {
            cn = new ConnectionManager();
        }

        public bool IsConnected()
        {
            return cn.IsConnected(MSAInstance);
        }
        #region Wrapper Functions

        /// <summary>
        /// Establish connection with HW
        /// </summary>
        /// <returns></returns>
        public bool Connect(string IP, int port)
        {
            return cn.Connect(MSAInstance, IP, port);
        }

        /// <summary>
        /// Disconnect from HW
        /// </summary>
        /// <returns></returns>
        public bool Disconnect()
        {
            return cn.Disconnect(MSAInstance);
        }

        public void SetTarget(byte target)
        {
            cn.SetTarget = target;
        }
        //public bool IsConnected(UInt16 MSAInstance)
        //{
        //  return cn.IsConnected(MSAInstance);
        //}

     


        /// <summary>
        /// Set  Port
        /// </summary>
        /// <param name="port"> port address</param>
        /// <param name="value">port value</param>
        /// <returns></returns>
        public bool SetPort(byte port, UInt16 value)
        {
            return cn.SetPort(MSAInstance, port, value);
        }

        /// <summary>
        /// Get  Port
        /// </summary>
        /// <param name="port"> port address</param>
        /// <param name="value">port value</param>
        /// <returns></returns>
        public UInt16 GetPort(byte port)
        {
            UInt16 value;
            value = cn.GetPort(MSAInstance, port);
            return value;
        }



        public int ReadBoardID()
        {
            return cn.ReadBoardID(MSAInstance);
        }




        public bool GetRev(ref UInt16 data)
        {
            data = cn.GetRev();
            if (data != null)
            {
                return true;
            }
            return false;
        }
        /// <summary>
        /// Read I2C
        /// </summary>
        /// <param name="registerAdress">Register Address</param>
        /// <returns></returns>
        public byte[] I2CRead(int registerAdress)
        {
            return cn.I2CRead(MSAInstance, registerAdress);
        }

        /// <summary>
        /// Write I2C
        /// </summary>
        /// <param name="registerAdress">Register Address</param>
        /// <param name="registerValue">data to be written</param>
        /// <returns></returns>
        public bool I2CWrite(int registerAdress, int registerValue)
        {
            return cn.I2CWrite(MSAInstance, registerAdress, registerValue);
        }


        #region AOC

        //public bool SetPE(byte SlaveAdd, byte Val)
        //{
        //    return cn.SetPE(MSAInstance, SlaveAdd, Val);
        //}

        //public bool GetPE(byte SlaveAdd, ref byte Val)
        //{
        //    return cn.GetPE(MSAInstance, SlaveAdd, ref Val);
        //}

        public bool EEPROM_Write(byte block, byte Address, byte Data)
        {
            return cn.EEPROM_Write(MSAInstance, block, Address, Data);
        }

        public bool EEPROM_Read(byte block, byte Address, ref byte Data)
        {
            return cn.EEPROM_Read(MSAInstance, block, Address, ref Data);
        }

        //ADC
        //public bool GetIntADC(byte ADCchannel, ref byte receive_buffer)
        //{
        //    return cn.GetIntADC(MSAInstance, ADCchannel, ref receive_buffer);
        //}

        //power
        public bool SetPower(byte rate)
        {
            return cn.SetPower(MSAInstance, rate);
        }

        //public bool SetFan_ON_OFF(byte Enable)
        //{
        //    return cn.SetFan_ON_OFF(MSAInstance, Enable);
        //}

        //public bool Set_PWM(byte channel, byte value)
        //{
        //    return cn.Set_PWM(MSAInstance, channel, value);
        //}

        //analog

        public double GetCurrent_sens()
        {
            double Data = 0;
            cn.GetCurrent_sens(MSAInstance, ref Data);
            return Data;
        }

        public double GetCurrent_sens_VCC1()
        {
            double Data = 0;
            cn.GetCurrent_sens_VCC1(MSAInstance, ref Data);
            return Data;
        }
        public double GetCurrent_sens_VCCTX()
        {
            double Data = 0;
            cn.GetCurrent_sens_VCCTX(MSAInstance, ref Data);
            return Data;
        }
        public double GetCurrent_sens_VCCRX()
        {
            double Data = 0;
            cn.GetCurrent_sens_VCCRX(MSAInstance, ref Data);
            return Data;
        }

        public double Get_VCC()
        {
            double Data = 0;
            cn.Get_VCC(MSAInstance, ref Data);
            return Data;
        }
        public double Get_VCC1()
        {
            double Data = 0;
            if (cn.Get_VCC1(MSAInstance, ref Data))
                return Data;
            else
                return Data;
        }
        public double Get_VCCTX()
        {
            double Data = 0;
            cn.Get_VCCTX(MSAInstance, ref Data);
            return Data;
        }

        public double Get_VCCRX()
        {
            double Data = 0;
            cn.Get_VCCRX(MSAInstance, ref Data);
            return Data;
        }

        public bool Get_temperature(ref double Data)
        {
            return cn.Get_temperature(MSAInstance, ref Data);
        }

        public bool Set_Module_VCC(float voltage)
        {
            return cn.Set_Module_VCC(MSAInstance, voltage);
        }

        public bool Set_Noise_Amplitude(byte amp)
        {
            return cn.Set_VCC_noise_amplitude(MSAInstance, amp);
        }

        public bool Set_VCC_Noise_frequency(Int32 Frequency)
        {
            return cn.Set_VCC_noise_frequency(MSAInstance, Frequency);
        }
        #endregion




        #endregion


        #region Bit Control

        /// <summary>
        /// Convert byte from a buffer of bytes into integer value
        /// </summary>
        /// <param name="tempReadBuff">byte vale</param>
        /// <param name="index">index of the byte to be converted to integer </param>
        /// <returns></returns>
        public int ReadBuffer(byte[] tempReadBuff, int index)
        {
            tempReadString = tempReadBuff[index].ToString("X2");
            return StringIntoValue(tempReadString);
        }

        /// <summary>
        /// convert a string of hex into interger value
        /// </summary>
        /// <param name="textvalue">string to be converted</param>
        /// <returns></returns>
        public int StringIntoValue(string textvalue)
        {
            return (Int32.Parse(textvalue, System.Globalization.NumberStyles.HexNumber));
        }

        /// <summary>
        /// Set a bit 
        /// </summary>
        /// <param name="val">value to set a bit inside it</param>
        /// <param name="bitToSet">bit to be set</param>
        public void SetBit(ref int val, BITS bitToSet)
        {
            val = (val | (byte)bitToSet);
        }

        /// <summary>
        /// clear a bit
        /// </summary>
        /// <param name="val">value to clear a bit inside it</param>
        /// <param name="bitToClr">bit to be cleared</param>
        public void ClearBit(ref int val, BITS bitToClr)
        {
            val = (val & ~(byte)bitToClr);
        }

        /// <summary>
        /// checks if a chosen bit is set or not
        /// </summary>
        /// <param name="val">value to check the bit inside it</param>
        /// <param name="bitToCheck">bit to be checked</param>
        /// <returns></returns>
        public bool IsBitSet(int val, BITS bitToCheck)
        {
            if ((val & (UInt16)bitToCheck) == (UInt16)bitToCheck)
            {
                return true;
            }
            return false;
        }


        /// <summary>
        /// convert a hexadecimal value into binary value
        /// </summary>
        /// <param name="hexvalue">hex value to be converted</param>
        /// <returns></returns>
        public string hex2binary(string hexvalue)
        {
            tempReadString = "";
            tempReadString = Convert.ToString(Convert.ToInt32(hexvalue, 16), 2);
            return tempReadString;
        }

        /// <summary>
        /// verifies the register address is in decimal format between 0 and 255
        /// </summary>
        /// <param name="textEntry">Entered address that must be checked</param>
        /// <returns></returns>
        public bool VerifyAddress(ref string textEntry)
        {
            int address = 0;
            if (textEntry.Length < 1)
            {
                return false;
            }
            else
            {
                address = Convert.ToInt32(textEntry);
                if ((address < 0) || (address > 255))
                {
                    return false;
                }
                return true;
            }
        }

        /// <summary>
        /// returns a string with a specific length
        /// </summary>
        /// <param name="str">string</param>
        /// <param name="length">desired length</param>
        /// <returns></returns>
        public string ReturnStringWithSpecificLength(string str, int length)
        {
            if (str.Length > length)
            {
                str = str.Remove(length, str.Length - length);
            }
            else if (str.Length < length)
            {
                for (int i = str.Length; i < length; i++)
                {
                    str = str.Insert(0, "0");
                }
            }
            return str;
        }

        /// <summary>
        /// force a string to be a hexadecimal string 
        /// </summary>
        /// <param name="str">string value</param>
        /// <returns></returns>
        public string HexString(string str)
        {
            for (int i = 0; i < str.Length; i++)
            {
                if ((str[i] < '0') || (str[i] > 'f'))
                {
                    str = str.Replace(str[i], '0');
                }
            }
            return str;
        }
        #endregion



        #region GUI Functions

        /// <summary>
        /// Check if the module is present or not, rD0
        /// </summary>
        /// <returns></returns>
        public bool IsModulePresent()
        {
            //tempPort = cn.GetPort(3);
            //if (IsBitSet(tempPort, BITS.BIT0))
            //{
            //    return false;
            //}
            //else
            //{
            //    return true;
            //}

            if (BoardID == 4062)
            {
                //HW Rev2
                if (cn.target == 0)
                {
                    tempPort = cn.GetPort(MSAInstance, 6); // RG12
                    if (IsBitSet(tempPort, BITS.BIT12))
                    {
                        return false;
                    }
                    else
                    {
                        return true;
                    }
                }
                else
                {
                    tempPort = cn.GetPort(MSAInstance, 4); // RE7
                    if (IsBitSet(tempPort, BITS.BIT7))
                    {
                        return false;
                    }
                    else
                    {
                        return true;
                    }
                }
            }else if(BoardID==4162)
            {
                tempPort = cn.GetPort(MSAInstance, 4); // RRE0
                if (IsBitSet(tempPort, BITS.BIT0))
                {
                    return false;
                }
                else
                {
                    return true;
                }
            }
            else
            {
                return false;
            }
        }
        public float TECSuplyCurrent2()
        {
            tempInt = ReadXBuffer(20, 2);
            return ((float)Calculate2sComplement16((ushort)tempInt) / 32767) * 100;  //32767->100%
        }
        public float SuplyVoltageMeasurements2()
        {
            tempInt = ReadXBuffer(23, 1);
            return (float)(tempInt) / 10000;  // the unit is in 100 uV so we divide by 10000 to make it in V
        }
        /// <summary>
        /// Select the module
        /// </summary>
        public void SelectModule()
        {
            //tempPort = cn.GetPort(1);  //port B,   RB3 Reset, RB7 ModSel

            //if (!IsBitSet(tempPort, BITS.BIT3))
            //{
            //    SetBit(ref tempPort, BITS.BIT3);
            //    ClearBit(ref tempPort, BITS.BIT7);
            //}

            //cn.SetPort(1, tempPort);

            //tempPort = cn.GetPort(3);  //port D,   RD1 Reset, 
            //int tempPort1 = 0; 
            //if (!IsBitSet(tempPort, BITS.BIT1))
            //{
            //    tempPort1 = cn.GetPort(1); //RB0 ModSel
            //    SetBit(ref tempPort, BITS.BIT0);
            //    ClearBit(ref tempPort1, BITS.BIT0);
            //    //SetBit(ref tempPort, BITS.BIT2);                 
            //    cn.SetPort((byte)1, (byte)tempPort1);
            //}
            //cn.SetPort((byte)3, (byte)tempPort);  

            if (BoardID == 4062)
            {
                //HW Rev 2

                cn.SetPort(MSAInstance, (byte)3, (byte)2);
            }
       
        }

        /// <summary>
        /// check for Software interrupts
        /// </summary>
        /// <returns></returns>
        public bool SWInterrupt()
        {
            //  INTL is portB at RB1
            tempPort = cn.GetPort(MSAInstance, 1); //port B
            if (IsBitSet(tempPort, BITS.BIT1))  // no interrupt is thrown by the mnodule
            {
                Thread.Sleep(10);
                return true;
            }
            return false;
        }
        //public bool Ping(ref FWInfo PingInfo)
        //{
        //    return cn.Ping(MSAInstance, ref PingInfo);
        //}
        //public int GetDeviceCount()
        //{
        //    return cn.GetDeviceCount();
        //}
        //public bool SetDeviceID(int D_ID)
        //{
        //    return cn.SetDeviceID(MSAInstance, (byte)D_ID);
        //}
        /// <summary>
        /// check for hardware interrupts
        /// </summary>
        /// <returns></returns>
        public bool HWInterrupt()
        {
            tempReadBuff1 = cn.I2CRead(MSAInstance, 2);  //bit 1 in byte 2 in Low Mem is for INTL
            tempReadString = tempReadBuff1[0].ToString("X2");
            tempInterval = StringIntoValue(tempReadString);
            if (IsBitSet(tempInterval, BITS.BIT1))  // no interrupt, INTL is high
            {
                Thread.Sleep(10);
                return true;
            }
            return false;
        }
        /// <summary>
        /// verify if there is a rate selection or not
        /// </summary>
        /// <returns></returns>
        public bool NoRateSelected()
        {
            cn.I2CWrite(MSAInstance, 127, 00);
            if (IsBitSet(ReadBuffer(cn.I2CRead(MSAInstance, 195), 0), BITS.BIT5))   //bit 5 of byte 195 page 00 is 0 implies no rate selection
            {
                return true;
            }
            else
            {
                return false;
            }
        }

        /// <summary>
        /// Check for extended rate selection
        /// </summary>
        /// <returns></returns>
        public bool ExtendedRateSelected()
        {
            if ((!IsBitSet(ReadBuffer(cn.I2CRead(MSAInstance, 221), 0), BITS.BIT2))
                && (IsBitSet(ReadBuffer(cn.I2CRead(MSAInstance, 221), 0), BITS.BIT3))
                && (IsBitSet(ReadBuffer(cn.I2CRead(MSAInstance, 141), 0), BITS.BIT0)))
            {
                return true;
            }
            else
            {
                return false;
            }
        }
        /// <summary>
        /// check for rate by aplication selection
        /// </summary>
        /// <returns></returns>
        public bool RateByApplicationSelected()
        {
            if ((IsBitSet(ReadBuffer(cn.I2CRead(MSAInstance, 221), 0), BITS.BIT2)) && (!IsBitSet(ReadBuffer(cn.I2CRead(MSAInstance, 221), 0), BITS.BIT3)))
            {
                return true;
            }
            else
            {
                return false;
            }
        }

        /// <summary>
        /// read a byte from HW and convert it to int
        /// </summary>
        /// <param name="address">byte address</param>
        /// <param name="index">index of that byte in the buffer of bytes</param>
        /// <returns></returns>
        public int ReadAddress(int address, int index)
        {
            return ReadBuffer(cn.I2CRead(MSAInstance, address), index);
        }

        public int ReadAddresses(int address, int index, ref byte[] ReadBuff)
        {
            ReadBuff = new byte[32];
            return ReadBuffer(cn.I2CReadd(MSAInstance, address, ref ReadBuff), index);
        }
        /// <summary>
        /// rerad multiple bytes form HW 
        /// </summary>
        /// <param name="address">start address </param>
        /// <param name="length">number of bytes to be read</param>
        /// <returns></returns>
        public ushort ReadXBuffer(int address, int length)
        {
            // change after v3.3 
            //tempReadBuff1 = cn.I2CRead(address);
            //tempReadString = "";
            //for (int i = 0; i < length; i++)
            //{
            //    tempReadString += tempReadBuff1[i].ToString("X2");
            //}
            //return tempReadString;

            byte lsb = 0;
            byte msb = 0;
            cn.I2CReadd(MSAInstance, address, ref tempReadBuff1);
            lsb = tempReadBuff1[1];
            msb = tempReadBuff1[0];
            ushort result = (ushort)((ushort)(msb << 8) | (ushort)lsb);
            return result;

        }

        /// <summary>
        /// read the bytes corresponding for tempearture measurement and calculate the equivalent temperature
        /// </summary>
        /// <returns></returns>
        public float TemperatureMeasurement(int address, int length)
        {
            tempInt = ReadXBuffer(address, length);
            return (float)Calculate2sComplement16((ushort)tempInt) / 256f;
        }


        /// <summary>
        /// Calculates the 2's Complement of a 16-bit value
        /// </summary>
        /// <param name="value"></param>
        /// <returns></returns>
        public int Calculate2sComplement16(ushort value)
        {
            bool IsNegative = (value & 0x8000) != 0;

            if (IsNegative)
                return (-(ushort)(~value + 1));
            else
                return value;
        }

        /// <summary>
        /// read the bytes corresponding for voltage measurement and calculate the equivalent voltage
        /// </summary>
        /// <returns></returns>
        public float SuplyVoltageMeasurements()
        {
            tempInt = ReadXBuffer(16, 2);
            return (float)(tempInt) / 10000;  // the unit is in 100 uV so we divide by 10000 to make it in V
        }

        /*public float SuplyVoltageMeasurements2()
        {
            tempInt = ReadXBuffer(23, 1);
            return (float)(tempInt) / 10000;  // the unit is in 100 uV so we divide by 10000 to make it in V
        }
        */
        public float SuplyCurrent()
        {
            tempInt = ReadXBuffer(28, 2);
            return (float)(tempInt) / 1000;  //lsb = 1mA
        }

        /// <summary>
        /// Read bytes corresponding to Squelsh disable
        /// </summary>
        /// <param name="address">byte address</param>
        /// <returns></returns>
        public int ReadSquelch(UInt16 address)
        {
            cn.I2CWrite(MSAInstance, 127, 0);
            return ReadBuffer(cn.I2CRead(MSAInstance, address), 0);
        }

        /// <summary>
        /// Control the check boxes according to the correspondingbit in the given address, in the low mem page
        /// </summary>
        /// <param name="Address">address</param>
        /// <param name="bit">corresponding bit </param>
        /// <param name="cbChecked"> check/unchek the checkbox</param>
        public void RxTxTempVccCheckBox(int Address, BITS bit, bool cbChecked)
        {

            tempInt = ReadAddress(Address, 0);
            if (cbChecked == true)
            {
                SetBit(ref tempInt, bit);
                I2CWrite(Address, tempInt);
            }
            else
            {
                ClearBit(ref tempInt, bit);
                I2CWrite(Address, tempInt);
            }
        }

        /// <summary>
        /// Control the check boxes according to the correspondingbit in the given address, in the upper pages
        /// </summary>
        /// <param name="Address">address</param>
        /// <param name="bit">corresponding bit </param>
        /// <param name="cbChecked"> check/unchek the checkbox</param>
        /// <param name="page">page number</param>
        public void RxTxTempVccCheckBox(int Address, BITS bit, bool cbChecked, int page)
        {
            I2CWrite(127, page);
            tempInt = ReadAddress(Address, 0);
            if (cbChecked == true)
            {
                SetBit(ref tempInt, bit);
                I2CWrite(Address, tempInt);
            }
            else
            {
                ClearBit(ref tempInt, bit);
                I2CWrite(Address, tempInt);
            }
        }

        /// <summary>
        /// Limited power settings
        /// </summary>
        /// <param name="Address">address to modify</param>
        /// <param name="bit1">first bit to be modified</param>
        /// <param name="bit2">second bit to be modified</param>
        /// <param name="cbChecked">check/uncheck the corresponding checkbox</param>
        public void LimitedPower(int Address, BITS bit1, BITS bit2, bool cbChecked)
        {
            tempInt = ReadAddress(Address, 0);

            if (cbChecked == true)
            {
                SetBit(ref tempInt, bit1);
                SetBit(ref tempInt, bit2); // bit: 1-0 are 11 means limited power
                I2CWrite(Address, tempInt);

            }
            else
            {
                ClearBit(ref tempInt, bit1);
                SetBit(ref tempInt, bit2);// bit: 1-0 are 01 means full power
                I2CWrite(Address, tempInt);

            }
        }

        /// <summary>
        /// full power settings
        /// </summary>
        /// <param name="Address">address to modify</param>
        /// <param name="bit1">first bit to be modified</param>
        /// <param name="bit2">second bit to be modified</param>
        /// <param name="cbChecked">check/uncheck the corresponding checkbox</param>
        public void FullPower(int Address, BITS bit1, BITS bit2, bool cbChecked)
        {
            tempInt = ReadAddress(Address, 0);

            if (cbChecked == true)
            {
                SetBit(ref tempInt, bit1);
                ClearBit(ref tempInt, bit2);// bit: 1-0 are 01 means full power
                I2CWrite(Address, tempInt);

            }
            else
            {
                SetBit(ref tempInt, bit1);
                SetBit(ref tempInt, bit2);// bit: 1-0 are 11 means limited power
                I2CWrite(Address, tempInt);

            }
        }

        /// <summary>
        /// Rx Application selection Settings
        /// </summary>
        /// <param name="Address">Address to be modified</param>
        /// <param name="str">value to be written into the corresponding address</param>
        public void RxApplicationSelect(int Address, string str)
        {
            str = ReturnStringWithSpecificLength(str, 2);
            str = HexString(str);
            I2CWrite(Address, Int32.Parse(str, System.Globalization.NumberStyles.HexNumber));
        }

        /// <summary>
        /// read multiple bytes from HW and convert them to ASCII code
        /// </summary>
        /// <param name="Address">byte address</param>
        /// <param name="length">number of bytes to be read</param>
        /// <param name="page">page number</param>
        /// <returns></returns>
        public string AscciiDisplay(int Address, int length, int page)
        {
            ReadString = "";
            asciiResult = "";

            I2CWrite(127, page);
            tempReadBuff1 = I2CRead(Address);

            for (int i = 0; i < length; i++)
            {
                ReadString += tempReadBuff1[i].ToString("X2");
                tempInt = StringIntoValue(tempReadBuff1[i].ToString("X2"));
                asciiResult += Convert.ToChar(tempInt);
            }
            return asciiResult;
        }

        /// <summary>
        /// returns the vendor OUI
        /// </summary>
        /// <returns></returns>
        public string vendorOUI()
        {
            ReadString = "";
            I2CWrite(127, 0);
            tempReadBuff1 = I2CRead(145);
            for (int i = 0; i < 3; i++)
            {
                ReadString += tempReadBuff1[i].ToString("X2");
            }
            return ReadString;
        }

        /// <summary>
        /// calculates the centered wavelength
        /// </summary>
        /// <returns></returns>
        public string CenteredWavelength()
        {
            ReadString = "";
            I2CWrite(127, 01);
            tempReadBuff1 = I2CRead(138);

            for (int i = 0; i < 2; i++)
            {
                ReadString += tempReadBuff1[i].ToString("X2");
            }
            tempInt = StringIntoValue(ReadString);
            tempInt = tempInt / 20;
            return tempInt.ToString();
        }
        /// <summary>
        /// calculates the wavelength tolerance
        /// </summary>
        /// <returns></returns>
        public string WavelengthTolerance()
        {
            ReadString = "";
            I2CWrite(127, 1);
            tempReadBuff1 = I2CRead(140);

            for (int i = 0; i < 2; i++)
            {
                ReadString += tempReadBuff1[i].ToString("X2");
            }

            tempInt = StringIntoValue(ReadString);
            tempInt = tempInt / 200;
            return tempInt.ToString();
        }
        /// <summary>
        /// calculates the checksum of bytes 128-190
        /// </summary>
        /// <returns></returns>
        public string CheckSum128190Val()
        {
            int sum = 0;
            string result = "";
            I2CWrite(127, 0);
            tempReadBuff1 = I2CRead(128);

            I2CWrite(127, 0);
            tempReadBuff2 = I2CRead(160);

            tempInt = 0;
            tempInterval = 0;
            //tempInt += StringIntoValue(tempReadBuff1[31].ToString("X2"));

            for (int i = 0; i < 32; i++)
            {
                tempInt += StringIntoValue(tempReadBuff1[i].ToString("X2"));
            }
            for (int i = 0; i < 31; i++)
            {
                tempInterval += StringIntoValue(tempReadBuff2[i].ToString("X2"));
            }

            sum = tempInterval + tempInt;

            result = sum.ToString("X4");
            return result.Substring(2, 2);
        }

        /// <summary>
        /// returns the vendor specific bytes
        /// </summary>
        /// <returns></returns>
        public string VendorSpecific()
        {
            ReadString = "";
            I2CWrite(127, 0);
            tempReadBuff1 = I2CRead(224);

            for (int i = 0; i < 32; i++)
            {
                ReadString += tempReadBuff1[i].ToString("X2");
            }
            return ReadString;
        }
        /// <summary>
        /// calculates the check sum of bytes 128-222
        /// </summary>
        /// <returns></returns>
        public string calc_CheckSum128_221val()
        {
            string Sum = "";
            byte[] tempArr = new byte[32];
            byte[] fullarray = new byte[96];
            tempInt = 0;

            I2CWrite(127, 0);
            for (int i = 0; i < 96; i += 32)
            {
                tempArr = I2CRead(128 + i);
                tempArr.CopyTo(fullarray, i);
            }
            for (int i = 0; i < 93; i++)
            {
                tempInt += StringIntoValue(fullarray[i].ToString("X2"));
            }
            Sum = tempInt.ToString("X4");
            return Sum.Substring(2, 2);
        }
        public int store_CheckSum128_221val()
        {
            byte[] tempArr = new byte[32];
            byte[] fullarray = new byte[96];
            tempInt = 0;

            I2CWrite(127, 0);
            for (int i = 0; i < 96; i += 32)
            {
                tempArr = I2CRead(128 + i);
                tempArr.CopyTo(fullarray, i);
            }
            for (int i = 0; i < 93; i++)
            {
                tempInt += StringIntoValue(fullarray[i].ToString("X2"));
            }
            I2CWrite(222, tempInt);
            return tempInt;
        }
        public string calc_CheckSum130_254valPage01()
        {
            string Sum = "";
            byte[] tempArr = new byte[32];
            byte[] fullarray = new byte[128];
            tempInt = 0;

            I2CWrite(127, 01);
            for (int i = 0; i < 128; i += 32)
            {
                tempArr = I2CRead(128 + i);
                tempArr.CopyTo(fullarray, i);
            }
            for (int i = 2; i < 127; i++)
            {
                tempInt += StringIntoValue(fullarray[i].ToString("X2"));
            }
            Sum = tempInt.ToString("X4");
            return Sum.Substring(2, 2);
        }
        public int store_CheckSum130_254valPage01()
        {
            byte[] tempArr = new byte[32];
            byte[] fullarray = new byte[128];
            tempInt = 0;

            I2CWrite(127, 01);
            for (int i = 0; i < 128; i += 32)
            {
                tempArr = I2CRead(128 + i);
                tempArr.CopyTo(fullarray, i);
            }
            for (int i = 2; i < 127; i++)
            {
                tempInt += StringIntoValue(fullarray[i].ToString("X2"));
            }
            I2CWrite(255, tempInt);
            return tempInt;
        }
        public string calc_CheckSum128_254valPage02()
        {
            string Sum = "";
            byte[] tempArr = new byte[32];
            byte[] fullarray = new byte[128];
            tempInt = 0;

            I2CWrite(127, 02);
            for (int i = 0; i < 128; i += 32)
            {
                tempArr = I2CRead(128 + i);
                tempArr.CopyTo(fullarray, i);
            }
            for (int i = 0; i < 127; i++)
            {
                tempInt += StringIntoValue(fullarray[i].ToString("X2"));
            }
            Sum = tempInt.ToString("X4");
            return Sum.Substring(2, 2);
        }
        public int store_CheckSum128_254valPage02()
        {
            byte[] tempArr = new byte[32];
            byte[] fullarray = new byte[128];
            tempInt = 0;

            I2CWrite(127, 02);
            for (int i = 0; i < 128; i += 32)
            {
                tempArr = I2CRead(128 + i);
                tempArr.CopyTo(fullarray, i);
            }
            for (int i = 0; i < 127; i++)
            {
                tempInt += StringIntoValue(fullarray[i].ToString("X2"));
            }
            I2CWrite(255, tempInt);
            return tempInt;
        }
        /// <summary>
        /// returns the module ID
        /// </summary>
        /// <returns></returns>
        public string ModuleID()
        {
            I2CWrite(127, 0);
            tempInt = ReadAddress(128, 0);
            Identifier id;
            id = (Identifier)tempInt;

            if ((tempInt >= 0x19) && (tempInt <= 0x7F))
            {
                return Identifier.RESERVED.ToString();
            }

            else if ((tempInt >= 0x80) && (tempInt <= 0xFF))
            {
                return Identifier.VENDOR_SPECIFIC.ToString();
            }

            else
            {
                return id.ToString();
            }
        }

        /// <summary>
        /// returns the connector ID
        /// </summary>
        /// <returns></returns>
        public string ConnectorID()
        {
            I2CWrite(127, 0);
            tempInt = ReadAddress(203, 0);
            Connector connectortype;
            connectortype = (Connector)tempInt;

            if (((tempInt >= 0x0E) && (tempInt <= 0x1F)) || ((tempInt >= 0x25) && (tempInt <= 0x7F)))
            {
                return Connector.Reserved.ToString();
            }

            else if ((tempInt >= 0x80) && (tempInt <= 0xFF))
            {
                return Connector.Vendor_specific.ToString();
            }

            else
            {
                return connectortype.ToString();
            }

        }
        /// <summary>
        /// returns the bite rate value
        /// </summary>
        /// <returns></returns>
        public string BitrateVal()
        {
            I2CWrite(127, 0);
            tempInt = ReadAddress(140, 0); //the bit rate is in unit of 100 MBps

            if (tempInt == 0xFF)
            {
                tempFloat = ReadAddress(222, 0);
                return (tempFloat * 250).ToString();
            }
            else
            {
                tempFloat = ReadAddress(140, 0);
                return (tempFloat * 100).ToString();
            }
        }

        /// <summary>
        /// reads multiple bytes and returns them as a concatenated string
        /// </summary>
        /// <param name="address">byte address</param>
        /// <param name="page">page number</param>
        /// <param name="bytenumber">number of bytes</param>
        /// <returns></returns>
        public string ReadxByte(int address, int page, int bytenumber)
        {
            I2CWrite(127, page);
            tempReadBuff1 = I2CRead(address);
            for (int i = 0; i < bytenumber; i++)
            {
                ReadString += tempReadBuff1[i].ToString("X2");
            }
            return ReadString.Substring(0, 2) + ReadString.Substring(2, 2);
        }
        #endregion


        public float TECSuplyCurrent1()
        {
            tempInt = ReadXBuffer(18, 2);
            return ((float)Calculate2sComplement16((ushort)tempInt) / 32767) * 100;  //32767->100%
        }

    }
}
